use std::collections::HashMap;

use nalgebra::Vector2;

use super::{block::Block, id::BlockId};


pub const CHUNK_WIDTH: usize = 32;
pub const CHUNK_HEIGHT: usize = 512;
const DIRT_LEVEL: u8 = 5;

pub struct Chunk {
    blocks: HashMap<(u8, u8), Block>,
}

impl Chunk {
    pub fn new_empty() -> Self {
        Self {
            blocks: HashMap::new()
        }
    }
    
    pub fn new_flat(surface_level: u8) -> Self {
        let mut w = Self::new_empty();
        for i in 0..CHUNK_WIDTH {
            w.add_block(BlockId::Grass, (i as u8, surface_level));
        }

        if surface_level < DIRT_LEVEL { return w; }

        for y in 1..=DIRT_LEVEL {
            for x in 0..CHUNK_WIDTH {
 
                let rv; 
                if y < 4 {
                    rv = 0.0
                } else {
                    rv = rand::random::<f32>() * (y / 2) as f32;
                }
                
                let id;
                if rv > 0.6 {
                    id = BlockId::Stone;
                } else {
                    id = BlockId::Dirt;
                }

                w.add_block(id, (x as u8, surface_level-y));
            }
        }

        for y in 1..surface_level-DIRT_LEVEL {
            for x in 0..CHUNK_WIDTH {
                w.add_block(BlockId::Stone, (x as u8, y));
            }
        }

        w
    }

    pub fn add_block(&mut self, id: BlockId, pos: (u8, u8)) {
        self.blocks.insert(pos, Block { id });
    }

     pub fn blocks(&self) -> &HashMap<(u8, u8), Block> {
        &self.blocks
    }

    pub fn blocks_mut(&mut self) -> &mut HashMap<(u8, u8), Block> {
        &mut self.blocks
    }
}

